@@ -54,6 +54,10 @@ class Overview(object):
         self.view_date = (dt.datetime.today() - dt.timedelta(hours = self.day_start.hour,
                                                         minutes = self.day_start.minute)).date()
 
+        self.range_pick = widgets.RangePick()
+        self.get_widget("range_pick_box").add(self.range_pick)
+        self.range_pick.connect("range-selected", self.on_range_selected)
+
         #set to monday
         self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1)
 
@@ -66,49 +70,21 @@ class Overview(object):
 
         self.end_date = self.start_date + dt.timedelta(6)
 
-
         self.overview = OverviewBox()
         self.get_widget("overview_tab").add(self.overview)
         self.fact_tree = self.overview.fact_tree # TODO - this is upside down, should maybe get the overview tab over here
         self.fact_tree.connect("cursor-changed", self.on_fact_selection_changed)
 
+        self.fact_tree.connect("button-press-event", self.on_fact_tree_button_press)
+
         self.reports = TotalsBox()
         self.get_widget("reports_tab").add(self.reports)
 
-        self.range_combo = gtk.combo_box_new_text()
-        self.ranges = {}
-        
-        def add_range_option(key, label):
-            """
-            Insert a new item in the range combo and map its index into the
-            ranges dictionary.
-            """
-            self.range_combo.append_text(label)
-            index = self.range_combo.get_model().iter_n_children(None)-1
-            self.ranges[key] = index
-
-        add_range_option("day", _("Day"))
-        add_range_option("week", _("Week"))
-        add_range_option("month", _("Month"))
-        add_range_option("custom", _("Date Range"))
-        
-        self.range_combo.set_active(self.ranges["week"])
-        self.range_combo.connect("changed", self.on_range_combo_changed)
-
-
-
-        self.get_widget("range_pick").add(self.range_combo)
-
-
-        self.start_date_input = widgets.DateInput(self.start_date)
-        self.start_date_input.connect("date-entered", self.on_start_date_entered)
-        self.get_widget("range_start_box").add(self.start_date_input)
-
-        self.end_date_input = widgets.DateInput(self.end_date)
-        self.end_date_input.connect("date-entered", self.on_end_date_entered)
-        self.get_widget("range_end_box").add(self.end_date_input)
+        self.current_range = "week"
 
         self.timechart = widgets.TimeChart()
+        self.timechart.connect("zoom-out-clicked", self.on_timechart_zoom_out_clicked)
+        self.timechart.connect("range-picked", self.on_timechart_new_range)
         self.timechart.day_start = self.day_start
 
         self.get_widget("by_day_box").add(self.timechart)
@@ -124,35 +100,64 @@ class Overview(object):
         else:
             window_box = conf.get("overview_window_box")
             if window_box:
-                x,y,w,h = (int(i) for i in window_box)
-                self.window.move(x, y)
+                x, y, w, h = (int(i) for i in window_box)
                 self.window.move(x, y)
                 self.window.resize(w, h)
             else:
                 self.window.set_position(gtk.WIN_POS_CENTER)
 
         self.window.show_all()
+
         self.search()
 
+    def on_fact_tree_button_press(self, treeview, event):
+        if event.button == 3:
+            x = int(event.x)
+            y = int(event.y)
+            time = event.time
+            pthinfo = treeview.get_path_at_pos(x, y)
+            if pthinfo is not None:
+                path, col, cellx, celly = pthinfo
+                treeview.grab_focus()
+                treeview.set_cursor( path, col, 0)
+                self.get_widget("fact_tree_popup").popup( None, None, None, event.button, time)
+            return True
+
+    def on_timechart_new_range(self, chart, start_date, end_date):
+        self.start_date = start_date
+        self.end_date = end_date
+        self.apply_range_select()
+
+    def on_timechart_zoom_out_clicked(self, chart):
+        if (self.end_date - self.start_date < dt.timedelta(days=6)):
+            self.on_week_activate(None)
+        elif (self.end_date - self.start_date < dt.timedelta(days=27)):
+            self.on_month_activate(None)
+        else:
+            self.current_range = "manual"
+            self.start_date = self.view_date.replace(day=1, month=1)
+            self.end_date = self.start_date.replace(year = self.start_date.year + 1) - dt.timedelta(days=1)
+            self.apply_range_select()
+
+
+
 
     def search(self):
         if self.start_date > self.end_date: # make sure the end is always after beginning
             self.start_date, self.end_date = self.end_date, self.start_date
 
-        self.start_date_input.set_date(self.start_date)
-        self.end_date_input.set_date(self.end_date)
-
         search_terms = self.get_widget("search").get_text().decode("utf8", "replace")
         self.facts = runtime.storage.get_facts(self.start_date, self.end_date, search_terms)
 
-        self.get_widget("report_button").set_sensitive(len(self.facts) > 0)
+        self.get_widget("export").set_sensitive(len(self.facts) > 0)
 
         self.set_title()
 
+        self.range_pick.set_range(self.start_date, self.end_date, self.view_date)
+
         durations = [(fact["start_time"], fact["delta"]) for fact in self.facts]
         self.timechart.draw(durations, self.start_date, self.end_date)
 
-
         if self.get_widget("window_tabs").get_current_page() == 0:
             self.overview.search(self.start_date, self.end_date, self.facts)
             self.reports.search(self.start_date, self.end_date, self.facts)
@@ -161,37 +166,8 @@ class Overview(object):
             self.overview.search(self.start_date, self.end_date, self.facts)
 
     def set_title(self):
-        start_date, end_date = self.start_date, self.end_date
-        dates_dict = stuff.dateDict(start_date, "start_")
-        dates_dict.update(stuff.dateDict(end_date, "end_"))
-
-        if start_date == end_date:
-            # date format for overview label when only single day is visible
-            # Using python datetime formatting syntax. See:
-            # http://docs.python.org/library/time.html#time.strftime
-            start_date_str = start_date.strftime(_("%B %d, %Y"))
-            # Overview label if looking on single day
-            self.title = start_date_str
-        elif start_date.year != end_date.year:
-            # overview label if start and end years don't match
-            # letter after prefixes (start_, end_) is the one of
-            # standard python date formatting ones- you can use all of them
-            # see http://docs.python.org/library/time.html#time.strftime
-            self.title = _(u"%(start_B)s %(start_d)s, %(start_Y)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
-        elif start_date.month != end_date.month:
-            # overview label if start and end month do not match
-            # letter after prefixes (start_, end_) is the one of
-            # standard python date formatting ones- you can use all of them
-            # see http://docs.python.org/library/time.html#time.strftime
-            self.title = _(u"%(start_B)s %(start_d)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
-        else:
-            # overview label for interval in same month
-            # letter after prefixes (start_, end_) is the one of
-            # standard python date formatting ones- you can use all of them
-            # see http://docs.python.org/library/time.html#time.strftime
-            self.title = _(u"%(start_B)s %(start_d)s – %(end_d)s, %(end_Y)s") % dates_dict
-
-        self.get_widget("range_title").set_text(self.title)
+        self.title = stuff.format_range(self.start_date, self.end_date)
+        self.window.set_title(self.title.decode("utf-8"))
 
 
     def on_conf_change(self, event, key, value):
@@ -226,7 +202,7 @@ class Overview(object):
         has_text = len(widget.get_text()) > 0
         widget.set_icon_sensitive(gtk.ENTRY_ICON_SECONDARY, has_text)
 
-    def on_report_button_clicked(self, widget):
+    def on_export_activate(self, widget):
         def on_report_chosen(widget, format, path):
             self.report_chooser = None
             reports.simple(self.facts, self.start_date, self.end_date, format, path)
@@ -234,8 +210,10 @@ class Overview(object):
             if format == ("html"):
                 webbrowser.open_new("file://%s" % path)
             else:
-                gtk.show_uri(gtk.gdk.Screen(),
-                             "file://%s" % os.path.split(path)[0], 0L)
+                try:
+                    gtk.show_uri(gtk.gdk.Screen(), "file://%s" % os.path.split(path)[0], 0L)
+                except:
+                    pass # bug 626656 - no use in capturing this one i think
 
         def on_report_chooser_closed(widget):
             self.report_chooser = None
@@ -250,100 +228,114 @@ class Overview(object):
             self.report_chooser.present()
 
 
+    def apply_range_select(self):
+        if self.view_date < self.start_date:
+            self.view_date = self.start_date
 
-    def on_range_combo_changed(self, combo):
-        idx = combo.get_active()
+        if self.view_date > self.end_date:
+            self.view_date = self.end_date
 
-        self.get_widget("preset_range").hide()
-        self.get_widget("range_box").hide()
-
-        if idx == self.ranges["custom"]:
-            self.get_widget("range_box").show()
-        else:
-            if idx == self.ranges["day"]:
-                self.start_date = self.view_date
-                self.end_date = self.start_date + dt.timedelta(0)
-                self.get_widget("preset_range").show()
-
-            elif idx == self.ranges["week"]:
-                self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1)
-                self.start_date = self.start_date + dt.timedelta(stuff.locale_first_weekday())
-                self.end_date = self.start_date + dt.timedelta(6)
-                self.get_widget("preset_range").show()
-
-            elif idx == self.ranges["month"]:
-                self.start_date = self.view_date - dt.timedelta(self.view_date.day - 1) #set to beginning of month
-                first_weekday, days_in_month = calendar.monthrange(self.view_date.year, self.view_date.month)
-                self.end_date = self.start_date + dt.timedelta(days_in_month - 1)
-                self.get_widget("preset_range").show()
+        self.search()
 
 
-            self.search()
+    def on_range_selected(self, widget, range, start, end):
+        self.current_range = range
+        self.start_date = start
+        self.end_date = end
+        self.apply_range_select()
+
+    def on_day_activate(self, button):
+        self.current_range = "day"
+        self.start_date = self.view_date
+        self.end_date = self.start_date + dt.timedelta(0)
+        self.apply_range_select()
+
+    def on_week_activate(self, button):
+        self.current_range = "week"
+        self.start_date, self.end_date = stuff.week(self.view_date)
+        self.apply_range_select()
+
+    def on_month_activate(self, button):
+        self.current_range = "month"
+        self.start_date, self.end_date = stuff.month(self.view_date)
+        self.apply_range_select()
+
+    def on_manual_range_apply_clicked(self, button):
+        self.current_range = "manual"
+        cal_date = self.get_widget("start_calendar").get_date()
+        self.start_date = dt.date(cal_date[0], cal_date[1] + 1, cal_date[2])
+
+        cal_date = self.get_widget("end_calendar").get_date()
+        self.end_date = dt.date(cal_date[0], cal_date[1] + 1, cal_date[2])
+
+        self.apply_range_select()
+
 
     def on_tabs_window_configure_event(self, window, event):
         # this is required so that the rows would grow on resize
         self.fact_tree.fix_row_heights()
 
 
-    def on_start_date_entered(self, input):
-        self.start_date = input.get_date()
-        self.view_date = self.start_date
-        self.search()
-
-    def on_end_date_entered(self, input):
-        self.end_date = input.get_date()
-        self.search()
-
-    def _chosen_range(self):
-        return self.range_combo.get_active()
-
-    def on_prev_clicked(self, button):
-        if self._chosen_range() == self.ranges["day"]:
+    def on_prev_activate(self, action):
+        if self.current_range == "day":
             self.start_date -= dt.timedelta(1)
             self.end_date -= dt.timedelta(1)
-        elif self._chosen_range() == self.ranges["week"]:
+        elif self.current_range == "week":
             self.start_date -= dt.timedelta(7)
             self.end_date -= dt.timedelta(7)
-        elif self._chosen_range() == self.ranges["month"]:
+        elif self.current_range == "month":
             self.end_date = self.start_date - dt.timedelta(1)
             first_weekday, days_in_month = calendar.monthrange(self.end_date.year, self.end_date.month)
             self.start_date = self.end_date - dt.timedelta(days_in_month - 1)
+        else:
+            # manual range - just jump to the next window
+            days =  (self.end_date - self.start_date) + dt.timedelta(days = 1)
+            self.start_date = self.start_date - days
+            self.end_date = self.end_date - days
 
         self.view_date = self.start_date
         self.search()
 
-    def on_next_clicked(self, button):
-        if self._chosen_range() == self.ranges["day"]:
+    def on_next_activate(self, action):
+        if self.current_range == "day":
             self.start_date += dt.timedelta(1)
             self.end_date += dt.timedelta(1)
-        elif self._chosen_range() == self.ranges["week"]:
+        elif self.current_range == "week":
             self.start_date += dt.timedelta(7)
             self.end_date += dt.timedelta(7)
-        elif self._chosen_range() == self.ranges["month"]:
+        elif self.current_range == "month":
             self.start_date = self.end_date + dt.timedelta(1)
             first_weekday, days_in_month = calendar.monthrange(self.start_date.year, self.start_date.month)
             self.end_date = self.start_date + dt.timedelta(days_in_month - 1)
+        else:
+            # manual range - just jump to the next window
+            days =  (self.end_date - self.start_date) + dt.timedelta(days = 1)
+            self.start_date = self.start_date + days
+            self.end_date = self.end_date + days
 
         self.view_date = self.start_date
         self.search()
 
 
-    def on_home_clicked(self, button):
+    def on_home_activate(self, action):
         self.view_date = (dt.datetime.today() - dt.timedelta(hours = self.day_start.hour,
                                                         minutes = self.day_start.minute)).date()
-        if self._chosen_range() == self.ranges["day"]:
+        if self.current_range == "day":
             self.start_date = self.view_date
             self.end_date = self.start_date + dt.timedelta(0)
 
-        elif self._chosen_range() == self.ranges["week"]:
+        elif self.current_range == "week":
             self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1)
             self.start_date = self.start_date + dt.timedelta(stuff.locale_first_weekday())
             self.end_date = self.start_date + dt.timedelta(6)
-
-        elif self._chosen_range() == self.ranges["month"]:
+        elif self.current_range == "month":
             self.start_date = self.view_date - dt.timedelta(self.view_date.day - 1) #set to beginning of month
             first_weekday, days_in_month = calendar.monthrange(self.view_date.year, self.view_date.month)
             self.end_date = self.start_date + dt.timedelta(days_in_month - 1)
+        else:
+            days =  (self.end_date - self.start_date)
+            self.start_date = self.view_date
+            self.end_date = self.view_date + days
 
         self.search()
 
@@ -360,7 +352,7 @@ class Overview(object):
             self.reports.do_charts()
 
 
-    def on_add_clicked(self, button):
+    def on_add_activate(self, action):
         fact = self.fact_tree.get_selected_fact()
         if not fact:
             selected_date = self.start_date
@@ -369,16 +361,17 @@ class Overview(object):
         else:
             selected_date = fact["date"]
 
-        dialogs.edit.show(fact_date = selected_date)
+        dialogs.edit.show(self, fact_date = selected_date)
 
-    def on_remove_clicked(self, button):
+    def on_remove_activate(self, button):
         self.overview.delete_selected()
 
-    def on_edit_clicked(self, button):
+
+    def on_edit_activate(self, button):
         fact = self.fact_tree.get_selected_fact()
         if not fact or isinstance(fact, dt.date):
             return
-        dialogs.edit.show(fact_id = fact["id"])
+        dialogs.edit.show(self, fact_id = fact["id"])
 
     def on_tabs_window_deleted(self, widget, event):
         self.close_window()
@@ -389,6 +382,9 @@ class Overview(object):
               and event_key.state & gtk.gdk.CONTROL_MASK)):
         self.close_window()
 
+    def on_close_activate(self, action):
+        self.close_window()
+
     def close_window(self):
         # properly saving window state and position
         maximized = self.window.get_window().get_state() & gtk.gdk.WINDOW_STATE_MAXIMIZED
