@@ -21,6 +21,7 @@
 import dbus, dbus.service
 import datetime as dt
 from calendar import timegm
+import gio
 
 def to_dbus_fact(fact):
     """Perform the conversion between fact database query and
@@ -47,6 +48,16 @@ class Storage(dbus.service.Object):
         dbus.service.Object.__init__(self, bus_name, self.__dbus_object_path__)
         self.mainloop = loop
 
+        self.__file = gio.File(__file__)
+        self.__monitor = self.__file.monitor_file()
+        self.__monitor.connect("changed", self._on_us_change)
+
+    # stop service when we have been updated (will be brought back in next call)
+    # anyway. should make updating simpler
+    def _on_us_change(self, monitor, gio_file, event_uri, event):
+        if event == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+            print "`%s` has changed. Quitting!" % __file__
+            self.Quit()
 
     def run_fixtures(self):
         pass
@@ -60,6 +71,9 @@ class Storage(dbus.service.Object):
     @dbus.service.signal("org.gnome.Hamster")
     def ActivitiesChanged(self): pass
 
+    @dbus.service.signal("org.gnome.Hamster")
+    def ToggleCalled(self): pass
+
     def dispatch_overwrite(self):
         self.TagsChanged()
         self.FactsChanged()
@@ -81,8 +95,15 @@ class Storage(dbus.service.Object):
         self.mainloop.quit()
 
 
-    # facts
+    @dbus.service.method("org.gnome.Hamster")
+    def Toggle(self):
+        """Toggle visibility of the main application window.
+           If several instances are available, it will toggle them all.
+        """
+        #log.logger.info("Hamster Service is being shutdown")
+        self.ToggleCalled()
 
+    # facts
     @dbus.service.method("org.gnome.Hamster", in_signature='ssiissb', out_signature='i')
     def AddFact(self, activity_name, tags, start_time, end_time,
                 category_name = None, description = None, temporary = False):
@@ -225,16 +246,9 @@ class Storage(dbus.service.Object):
         self.ActivitiesChanged()
 
 
-    @dbus.service.method("org.gnome.Hamster", out_signature='aa{sv}')
+    @dbus.service.method("org.gnome.Hamster", out_signature='a(is)')
     def GetCategories(self):
-        res = []
-        for category in self.__get_categories():
-            category = dict(category)
-            category['color_code'] = category['color_code'] or ''
-            res.append(category)
-
-
-        return res
+        return [(category['id'], category['name']) for category in self.__get_categories()]
 
 
     # activities
@@ -259,34 +273,19 @@ class Storage(dbus.service.Object):
         self.ActivitiesChanged()
         return result
 
-    @dbus.service.method("org.gnome.Hamster", in_signature='i', out_signature='aa{sv}')
+    @dbus.service.method("org.gnome.Hamster", in_signature='i', out_signature='a(isis)')
     def GetCategoryActivities(self, category_id = -1):
-        res = []
-        for activity in self.__get_category_activities(category_id = category_id):
-            activity = dict(activity)
-            activity['category'] = activity['category'] or ''
 
-            res.append(activity)
-
-        return res
+        return [(row['id'],
+                 row['name'],
+                 row['category_id'],
+                 row['name'] or '') for row in
+                      self.__get_category_activities(category_id = category_id)]
 
 
-    @dbus.service.method("org.gnome.Hamster", in_signature='s', out_signature='aa{sv}')
+    @dbus.service.method("org.gnome.Hamster", in_signature='s', out_signature='a(ss)')
     def GetActivities(self, search = ""):
-        res = []
-        for activity in self.__get_activities(search):
-            activity = dict(activity)
-            activity['category'] = activity['category'] or ''
-            res.append(activity)
-
-        return res
-
-
-
-    @dbus.service.method("org.gnome.Hamster", in_signature='iib')
-    def MoveActivity(self, source_id, target_order, insert_after = True):
-        self.__move_activity(source_id, target_order, insert_after)
-        self.ActivitiesChanged()
+        return [(row['name'], row['category'] or '') for row in self.__get_activities(search)]
 
 
     @dbus.service.method("org.gnome.Hamster", in_signature='ii', out_signature = 'b')
@@ -297,12 +296,6 @@ class Storage(dbus.service.Object):
         return changed
 
 
-    @dbus.service.method("org.gnome.Hamster", in_signature='iiii')
-    def SwapActivities(self, id1, priority1, id2, priority2):
-        self.__swap_activities(id1, priority1, id2, priority2)
-        self.ActivitiesChanged()
-
-
     @dbus.service.method("org.gnome.Hamster", in_signature='sib', out_signature='a{sv}')
     def GetActivityByName(self, activity, category_id, resurrect = True):
         category_id = category_id or None
@@ -313,17 +306,17 @@ class Storage(dbus.service.Object):
             return {}
 
     # tags
-    @dbus.service.method("org.gnome.Hamster", in_signature='b', out_signature='aa{sv}')
+    @dbus.service.method("org.gnome.Hamster", in_signature='b', out_signature='a(isb)')
     def GetTags(self, only_autocomplete):
-        return [dict(tag) for tag in self.__get_tags(only_autocomplete)]
+        return [(tag['id'], tag['name'], tag['autocomplete']) for tag in self.__get_tags(only_autocomplete)]
 
 
-    @dbus.service.method("org.gnome.Hamster", in_signature='as', out_signature='aa{sv}')
+    @dbus.service.method("org.gnome.Hamster", in_signature='as', out_signature='a(isb)')
     def GetTagIds(self, tags):
         tags, new_added = self.__get_tag_ids(tags)
         if new_added:
             self.TagsChanged()
-        return [dict(tag) for tag in tags]
+        return [(tag['id'], tag['name'], tag['autocomplete']) for tag in tags]
 
 
     @dbus.service.method("org.gnome.Hamster", in_signature='s')
