@@ -17,13 +17,13 @@
 # You should have received a copy of the GNU General Public License
 # along with Project Hamster.  If not, see <http://www.gnu.org/licenses/>.
 
-import gtk, gobject
+import gtk, gobject, pango
 import datetime as dt
 
-from .hamster.configuration import runtime
-
-from .hamster import stuff
-from .hamster.stuff import format_duration
+from ..configuration import runtime
+from .. import stuff, graphics
+from ..stuff import format_duration
+from .. import external
 
 class ActivityEntry(gtk.Entry):
     __gsignals__ = {
@@ -35,9 +35,11 @@ class ActivityEntry(gtk.Entry):
         gtk.Entry.__init__(self)
         self.news = False
         self.activities = None
+        self.external_activities = [] # suggestions from outer space
         self.categories = None
         self.filter = None
         self.max_results = 10 # limit popup size to 10 results
+        self.external = external.ActivitiesSource()
 
         self.popup = gtk.Window(type = gtk.WINDOW_POPUP)
 
@@ -49,21 +51,17 @@ class ActivityEntry(gtk.Entry):
         self.tree.set_headers_visible(False)
         self.tree.set_hover_selection(True)
 
-        bgcolor = gtk.Style().bg[gtk.STATE_NORMAL].to_string()
-        time_cell = gtk.CellRendererPixbuf()
-        time_cell.set_property("icon-name", "appointment-new")
-        time_cell.set_property("cell-background", bgcolor)
+        self.time_icon_cell = gtk.CellRendererPixbuf()
+        self.time_icon_cell.set_property("icon-name", "appointment-new")
 
-        self.time_icon_column = gtk.TreeViewColumn("",
-                                              time_cell)
+        self.time_icon_column = gtk.TreeViewColumn("", self.time_icon_cell)
         self.tree.append_column(self.time_icon_column)
 
-        time_cell = gtk.CellRendererText()
-        time_cell.set_property("scale", 0.8)
-        time_cell.set_property("cell-background", bgcolor)
+        self.time_cell = gtk.CellRendererText()
+        self.time_cell.set_property("scale", 0.8)
 
         self.time_column = gtk.TreeViewColumn("Time",
-                                              time_cell,
+                                              self.time_cell,
                                               text = 3)
         self.tree.append_column(self.time_column)
 
@@ -74,8 +72,13 @@ class ActivityEntry(gtk.Entry):
         self.activity_column.set_expand(True)
         self.tree.append_column(self.activity_column)
 
+        self.category_cell = gtk.CellRendererText()
+        self.category_cell.set_property('alignment', pango.ALIGN_RIGHT)
+        self.category_cell.set_property('scale', pango.SCALE_SMALL)
+        self.category_cell.set_property('yalign', 0.0)
+
         self.category_column = gtk.TreeViewColumn("Category",
-                                                  stuff.CategoryCell(),
+                                                  self.category_cell,
                                                   text=2)
         self.tree.append_column(self.category_column)
 
@@ -91,13 +94,35 @@ class ActivityEntry(gtk.Entry):
         self.connect("key-release-event", self._on_key_release_event)
         self.connect("focus-out-event", self._on_focus_out_event)
         self.connect("changed", self._on_text_changed)
+        self._parent_click_watcher = None # bit lame but works
 
         runtime.storage.connect('activities-changed',self.after_activity_update)
 
         self.show()
         self.populate_suggestions()
 
+    def get_value(self):
+        activity_name = self.get_text().decode("utf-8").strip()
+        if not activity_name:
+            return None, False
+
+        # see if entered text matches something from the outer suggestions
+        # only consequence of if it does is that it will not attempt to
+        # ressurect the activity if it's deleted (hidden)
+        # thus avoiding polluting our local suggestions
+        external_names = set()
+        for activity in self.external_activities:
+            name = activity['name']
+            if activity['category']:
+                name = "%s@%s" % name, activity['category']
+            external_names.add(name.lower())
+
+        return activity_name, activity_name in external_names
+
     def hide_popup(self):
+        if self._parent_click_watcher and self.get_toplevel().handler_is_connected(self._parent_click_watcher):
+            self.get_toplevel().disconnect(self._parent_click_watcher)
+            self._parent_click_watcher = None
         self.popup.hide()
 
     def show_popup(self):
@@ -106,6 +131,9 @@ class ActivityEntry(gtk.Entry):
             self.hide_popup()
             return
 
+        if not self._parent_click_watcher:
+            self._parent_click_watcher = self.get_toplevel().connect("button-press-event", self._on_focus_out_event)
+
         activity = stuff.parse_activity_input(self.filter)
         time = ''
         if activity.start_time:
@@ -120,6 +148,16 @@ class ActivityEntry(gtk.Entry):
         self.category_column.set_visible(self.filter.find("@") == -1)
 
 
+        #set proper background color (we can do that only on a realised widget)
+        bgcolor = self.get_style().bg[gtk.STATE_NORMAL]
+        self.time_icon_cell.set_property("cell-background", bgcolor)
+        self.time_cell.set_property("cell-background", bgcolor)
+
+        text_color = self.get_style().text[gtk.STATE_NORMAL]
+        category_color = graphics.Colors.contrast(text_color,  100)
+        self.category_cell.set_property('foreground-gdk', graphics.Colors.gdk(category_color))
+
+
         #move popup under the widget
         alloc = self.get_allocation()
 
@@ -184,7 +222,10 @@ class ActivityEntry(gtk.Entry):
         input_activity = stuff.parse_activity_input(self.filter)
 
         # do not cache as ordering and available options change over time
-        self.activities = runtime.storage.get_autocomplete_activities(input_activity.activity_name)
+        self.activities = runtime.storage.get_activities(input_activity.activity_name)
+        self.external_activities = self.external.get_activities(input_activity.activity_name)
+        self.activities.extend(self.external_activities)
+
         self.categories = self.categories or runtime.storage.get_categories()
 
 
